#ifndef DEBUGVIEW_H_INCLUDED
#define DEBUGVIEW_H_INCLUDED

#include <QWidget>
#include <QPainter>
#include <QVector>
#include <QVariant>
#include <QLineEdit>
#include "compat.h"
#include "DebugObject.h"
#include "emubase/Emubase.h"

class DebugView: public QWidget {
  Q_OBJECT
public:
  DebugView( QWidget *parent = 0 );
  void setCurrentProc( BOOL okCPU );
  void onUpdate();

protected:
  QVector <DebugObject> objects;
  int objIdx, selectedObj;
  QLineEdit *valueEditor;

  bool event( QEvent *e );
  bool eventFilter( QObject *obj, QEvent *ev );
  void paintEvent( QPaintEvent *e );
  void updateFontMetrics();
  void leaveEvent( QEvent *e );
  void mouseMoveEvent( QMouseEvent *e );
  void mouseReleaseEvent( QMouseEvent *e );
  void DrawOctalValue( QPainter& p, int x, int y, WORD value );
  void DrawBinaryValue( QPainter& p, int x, int y, WORD value );
  void setChangeHandler( const char *slot, QVariant p );
  void setRegister( int regNum, int newValue );

  BOOL m_okDebugProcessor;  // TRUE - CPU, FALSE - PPU
  WORD m_wDebugCpuR[9];  // Old register values - R0..R7, PSW
  WORD m_wDebugPpuR[9];  // Old register values - R0..R7, PSW
  BOOL m_okDebugCpuRChanged[9];  // Register change flags
  BOOL m_okDebugPpuRChanged[9];  // Register change flags
  int cxChar, cyLine, ascent;

  void drawProcessor( QPainter& p, CProcessor* pProc, int x, int y, WORD* arrR, BOOL* arrRChanged );
  void drawDisassemble( QPainter& p, CProcessor* pProc, WORD previous, int x, int y );
  void drawMemoryForRegister( QPainter& p, int reg, CProcessor* pProc, int x, int y );
  void drawPorts( QPainter& p, BOOL okProcessor, CMemoryController* pMemCtl, CMotherboard* pBoard, int x, int y );
  void drawChannels( QPainter& p, int x, int y );

protected slots:
  void showMenu();
  void debugCPU();
  void debugPPU();
  void setRegisterOctal( QVariant p, QString newValue );
  void setRegisterBinary( QVariant p, QString newValue );
  void setMemoryValue( QVariant p, QString newValue );
  void changeValue();
};

#endif // DEBUGVIEW_H_INCLUDED
