#ifndef MEMORYVIEW_H_INCLUDED
#define MEMORYVIEW_H_INCLUDED

#include <QWidget>
#include <QScrollBar>
#include <QHBoxLayout>
#include <QWheelEvent>
#include <QApplication>

#include "compat.h"
#include "DebugObject.h"

enum MemoryViewMode {
    MEMMODE_RAM0 = 0,  // RAM plane 0
    MEMMODE_RAM1 = 1,  // RAM plane 1
    MEMMODE_RAM2 = 2,  // RAM plane 2
    MEMMODE_ROM  = 3,  // ROM
    MEMMODE_CPU  = 4,  // CPU memory
    MEMMODE_PPU  = 5,  // PPU memory
    MEMMODE_LAST = 5,  // Last mode number
};

class MemoryView0: public QWidget {
  Q_OBJECT
public:
  MemoryView0( QScrollBar *sb, QWidget *parent = 0 );

protected:
  QScrollBar *scrollBar;
  QVector <DebugObject> objects;
  int objIdx, selectedObj;

  bool event( QEvent *e );
  void paintEvent( QPaintEvent *e );
  void updateFontMetrics();
  void leaveEvent( QEvent *e );
  void mouseMoveEvent( QMouseEvent *e );
  void mousePressEvent( QMouseEvent *e );
  void DrawOctalValue( QPainter& p, int x, int y, WORD value );
  void DrawBinaryValue( QPainter& p, int x, int y, WORD value );
  void setChangeHandler( const char *slot, QVariant p );

  int m_cyLineMemory; // Line height in pixel
  int m_Mode;
  WORD m_wBaseAddress;
  int cxChar, cyLine, ascent;

  LPCTSTR getMemoryModeName();
  void scrollTo(WORD wAddress);

  void showMem( int mode ) {
    m_Mode = mode;
    repaint();
  }

  void wheelEvent( QWheelEvent *ev ) {
  	QApplication::sendEvent( scrollBar, ev );
  }

protected slots:
  void showMenu();
  void showRAM0() { showMem( MEMMODE_RAM0 ); }
  void showRAM1() { showMem( MEMMODE_RAM1 ); }
  void showRAM2() { showMem( MEMMODE_RAM2 ); }
  void showROM()  { showMem( MEMMODE_ROM ); }
  void showCPU()  { showMem( MEMMODE_CPU ); }
  void showPPU()  { showMem( MEMMODE_PPU ); }
  void selectAddress();
  void setMemoryValue( QVariant p, QString newValue );
};

class MemoryView: public QWidget {
public:
  MemoryView( QWidget *parent = 0 ): QWidget ( parent ) {
	QScrollBar *scrollBar = new QScrollBar( Qt::Vertical );
	MemoryView0 *mv0 = new MemoryView0( scrollBar );

	QHBoxLayout *layout = new QHBoxLayout();
	layout->addWidget( mv0 );
	layout->addWidget( scrollBar );

	setLayout( layout );
	show();
  }
};

#endif // MEMORYVIEW_H_INCLUDED
